package com.xs.core.config.httpclient;

import org.apache.hc.client5.http.config.RequestConfig;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.client5.http.impl.io.PoolingHttpClientConnectionManager;
import org.apache.hc.client5.http.socket.ConnectionSocketFactory;
import org.apache.hc.client5.http.socket.PlainConnectionSocketFactory;
import org.apache.hc.client5.http.ssl.NoopHostnameVerifier;
import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactory;
import org.apache.hc.core5.http.config.Registry;
import org.apache.hc.core5.http.config.RegistryBuilder;
import org.apache.hc.core5.ssl.SSLContexts;
import org.apache.hc.core5.util.Timeout;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import javax.net.ssl.SSLContext;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;

@Configuration
public class HttpClientConfig {
    @Bean
    @Scope("prototype")//设定多例对象
    public CloseableHttpClient httpClient(@Qualifier("httpClientConnectionManager") PoolingHttpClientConnectionManager connectionManager) {
        // 创建并返回一个自定义配置的 CloseableHttpClient 实例
        return HttpClients.custom()
                // 设置连接管理器，管理连接池
                .setConnectionManager(connectionManager)
                // 设置默认请求配置
                .setDefaultRequestConfig(
                        RequestConfig.custom()
                                // 设置连接超时时间为 30 秒
                                .setConnectTimeout(Timeout.ofSeconds(30))
                                // 设置响应超时时间为 30 秒
                                .setResponseTimeout(Timeout.ofSeconds(30))
                                .build())
                .build();
    }

    /**
     * 连接池
     *
     * @return
     */
    @Bean("httpClientConnectionManager")
    public PoolingHttpClientConnectionManager connectionManager() {
        try {
            // 创建一个信任所有证书的自定义 SSLContext
            SSLContext sslContext = SSLContexts.custom().loadTrustMaterial(null, (chain, authType) -> true).build();

            // 创建 SSLConnectionSocketFactory，使用 NoopHostnameVerifier 来跳过主机名验证
            SSLConnectionSocketFactory sslSocketFactory = new SSLConnectionSocketFactory(sslContext, new String[]{"TLSv1", "TLSv1.1", "TLSv1.2"}, null, NoopHostnameVerifier.INSTANCE);
            // 注册 HTTP 和 HTTPS 套接字工厂
            Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
                    .register("http", PlainConnectionSocketFactory.INSTANCE)
                    .register("https", sslSocketFactory)
                    .build();

            PoolingHttpClientConnectionManager connectionManager = new PoolingHttpClientConnectionManager(registry);
            connectionManager.setMaxTotal(100);
            connectionManager.setDefaultMaxPerRoute(20);
            return connectionManager;
        } catch (KeyManagementException | NoSuchAlgorithmException | KeyStoreException e) {
            throw new RuntimeException("创建连接管理器时出错", e);
        }
    }

}
