package com.xs.core.controller.competition;

import com.xs.core.common.annotation.OperationLog;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.model.ResponseResult;
import com.xs.core.model.user.req.GoldCoinCompetitionRankingReq;
import com.xs.core.model.user.resp.GoldCoinLeaderboardInfo;
import com.xs.core.model.user.resp.TeamCompetitionResp;
import com.xs.core.model.user.resp.UserCompetitionResp;
import com.xs.core.service.competition.CompetitionService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.concurrent.CompletableFuture;

@RestController
@RequestMapping("/competition")
@Tag(name = "竞赛排行榜")
public class CompetitionController {
    @Autowired
    private CompetitionService competitionService;

    @Operation(summary = "个人金币排行榜", description = "个人金币排行榜")
    @PostMapping("/goldCoinCompetitionRanking")
    @OperationLog(description = "个人金币排行榜", operationType = "个人金币排行榜", module = "竞赛排行榜")
    public ResponseResult<GoldCoinLeaderboardInfo> goldCoinCompetitionRanking(@RequestBody GoldCoinCompetitionRankingReq query) {
        UserContext userContext = UserContextHolder.getContext();
        CompletableFuture<List<UserCompetitionResp>> listCompletableFuture = CompletableFuture.supplyAsync(() -> competitionService.individualGoldCoinCompetitionRanking(query.getLimit()));
        CompletableFuture<UserCompetitionResp> userCompetitionRespCompletableFuture = CompletableFuture.supplyAsync(() -> competitionService.getIndividualGoldCoinCompetitionInfo(userContext.getId()));
        CompletableFuture.allOf(listCompletableFuture, userCompetitionRespCompletableFuture);
        GoldCoinLeaderboardInfo goldCoinLeaderboardInfo = new GoldCoinLeaderboardInfo(listCompletableFuture.join(), userCompetitionRespCompletableFuture.join());
        return ResponseResult.success(goldCoinLeaderboardInfo);
    }

    @Operation(summary = "团队邀请排行榜", description = "团队邀请排行榜")
    @PostMapping("/teamInviteCompetitionRanking")
    @OperationLog(description = "团队邀请排行榜", operationType = "团队邀请排行榜", module = "竞赛排行榜")
    public ResponseResult<List<TeamCompetitionResp>> teamInviteCompetitionRanking(@RequestBody GoldCoinCompetitionRankingReq query) {
        List<TeamCompetitionResp> teamCompetitionResp = competitionService.teamInviteCompetitionRanking(query.getLimit(), 500);
        return ResponseResult.success(teamCompetitionResp);
    }
}
