package com.xs.core.controller.task;

import com.xs.core.common.annotation.OperationLog;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.model.ResponseResult;
import com.xs.core.model.task.req.TaskInfoReq;
import com.xs.core.model.task.resp.TaskInfoResp;
import com.xs.core.service.task.TaskService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Locale;

@RestController
@RequestMapping("/task")
@Tag(name = "任务信息", description = "任务信息")
@Slf4j
public class TaskInfoController {
    @Autowired
    private TaskService taskService;

    @Operation(description = "任务信息查询")
    @PostMapping("/getTaskInfoList")
    @OperationLog(description = "任务信息查询", operationType = "任务信息查询", module = "任务信息")
    public ResponseResult<List<TaskInfoResp>> getTaskInfoList(@RequestBody TaskInfoReq req) {
        Locale locale = LocaleContextHolder.getLocale();
        String languageTag = locale.toLanguageTag();
        req.setLang(languageTag);
        List<TaskInfoResp> taskInfoList = taskService.getTaskInfoList(req);
        return ResponseResult.success(taskInfoList);
    }

    @Operation(description = "处理任务")
    @PostMapping("/handleTask")
    @OperationLog(description = "处理任务", operationType = "处理任务", module = "任务信息")
    public ResponseResult<?> handleTask(@RequestBody TaskInfoReq req) {
        UserContext context = UserContextHolder.getContext();
        taskService.handleTask(req, context.getId());
        return ResponseResult.success();
    }
}
