package com.xs.core.controller.team;

import com.xs.core.common.annotation.OperationLog;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.content.UserContextHolder;
import com.xs.core.model.ResponseResult;
import com.xs.core.model.team.req.ClaimTeamInviteTaskRewardReq;
import com.xs.core.model.team.req.TeamShareReq;
import com.xs.core.model.team.resp.*;
import com.xs.core.service.team.TeamShareService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/team/share")
@Tag(name = "团队邀请", description = "团队邀请")
public class TeamShareController {
    @Autowired
    private TeamShareService teamShareService;

    @Operation(summary = "获取我的团队信息", description = "获取我的团队信息")
    @PostMapping("/getMyTeamsInfo")
    @OperationLog(description = "获取我的团队信息", operationType = "获取我的团队信息", module = "团队邀请")
    public ResponseResult<TeamsInfoResp> getMyTeamsInfo() {
        UserContext userContext = UserContextHolder.getContext();
        TeamsInfoResp teamsInfoResp = teamShareService.getMyTeamsInfo(userContext.getId());
        return ResponseResult.success(teamsInfoResp);
    }

    @Operation(summary = "获取团队产出奖励信息", description = "获取团队产出奖励信息")
    @PostMapping("/getTeamShareUserListByType")
    @OperationLog(description = "获取团队产出奖励信息", operationType = "获取团队产出奖励信息", module = "团队邀请")
    public ResponseResult<TeamShareInfoResp> getTeamShareUserListByType(@Validated @RequestBody TeamShareReq req) {
        UserContext userContext = UserContextHolder.getContext();
        TeamShareInfoResp teamShareUserInfo = teamShareService.getTeamShareUserInfoByType(userContext.getId(), req);
        return ResponseResult.success(teamShareUserInfo);
    }

    @Operation(summary = "收取团队奖励", description = "收取团队奖励")
    @PostMapping("/claimTeamShareReward")
    @OperationLog(description = "收取团队奖励", operationType = "收取团队奖励", module = "团队邀请")
    public ResponseResult<List<TeamShareUserResp>> claimTeamShareReward(@Validated @RequestBody TeamShareReq req) {
        UserContext userContext = UserContextHolder.getContext();
        teamShareService.claimTeamShareReward(userContext.getId(), req);
        return ResponseResult.success();
    }

    @Operation(summary = "获取团队邀请固定奖励列表", description = "获取团队邀请固定奖励列表(邀请一人奖励1000金币)")
    @PostMapping("/getTeamInviteRewardsList")
    @OperationLog(description = "获取团队邀请固定奖励列表", operationType = "获取团队邀请固定奖励列表", module = "团队邀请")
    public ResponseResult<List<TeamInviteRewardsResp>> getTeamInviteRewardsList() {
        UserContext userContext = UserContextHolder.getContext();
        List<TeamInviteRewardsResp> teamInviteRewardsList = teamShareService.getTeamInviteRewardsList(userContext.getId());
        return ResponseResult.success(teamInviteRewardsList);
    }

    @Operation(summary = "收取团队邀请固定奖励", description = "收取团队邀请固定奖励")
    @PostMapping("/claimTeamInviteReward")
    @OperationLog(description = "收取团队邀请固定奖励", operationType = "收取团队邀请固定奖励", module = "团队邀请")
    public ResponseResult<?> claimTeamInviteReward() {
        UserContext userContext = UserContextHolder.getContext();
        teamShareService.claimTeamInviteReward(userContext.getId());
        return ResponseResult.success();
    }

    @Operation(summary = "获取团队邀请任务可领取奖励", description = "获取团队邀请任务可领取奖励")
    @PostMapping("/getTeamInviteRewardsRules")
    @OperationLog(description = "获取团队邀请任务可领取奖励", operationType = "获取团队邀请任务可领取奖励", module = "团队邀请")
    public ResponseResult<TeamInviteAvailableRewardsResp> getTeamInviteAvailableRewardsRules() {
        UserContext userContext = UserContextHolder.getContext();
        TeamInviteAvailableRewardsResp resp = teamShareService.getTeamInviteAvailableRewardsRules(userContext.getId());
        return ResponseResult.success(resp);
    }

    @Operation(summary = "领取团队邀请任务奖励", description = "领取团队邀请任务奖励")
    @PostMapping("/claimTeamInviteTaskReward")
    @OperationLog(description = "领取团队邀请任务奖励", operationType = "领取团队邀请任务奖励", module = "团队邀请")
    public ResponseResult<?> claimTeamInviteTaskReward(@RequestBody ClaimTeamInviteTaskRewardReq req) {
        UserContext userContext = UserContextHolder.getContext();
        teamShareService.claimTeamInviteTaskReward(userContext.getId(), req.getRuleId());
        return ResponseResult.success();
    }
}
