package com.xs.core.init;

import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.ResponseResult;
import com.xs.core.utils.OpenApiEncryptionUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.sql.Connection;

@Component
@Slf4j
public class ProjectInitializer implements CommandLineRunner {
    @Value("${openapi.aes-key}")
    private String aesKey;
    @Value("${openapi.aes-iv}")
    private String aesIv;

    @Autowired
    private DataSource dataSource;
    @Autowired
    private MessageSource messageSource;

    @Override
    public void run(String... args) throws Exception {
        ResponseResult.setMessageSource(messageSource);
        CheckUtils.setMessageSource(messageSource);
        log.info("MessageSource initialized successfully.");
        OpenApiEncryptionUtils.initAes(aesKey, aesIv);
        log.info("Aes initialized successfully.");
        try (Connection connection = dataSource.getConnection()) {
            // 执行一个简单的查询来初始化连接
            connection.createStatement().execute("SELECT 1 from dual");
            log.info("Database connection initialized successfully.");
            log.info("tg-game startup successfully");
        }
    }
}
