package com.xs.core.mq.consumer;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjUtil;
import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.xs.core.common.enums.CoinTransactionCategoryEnum;
import com.xs.core.common.enums.CoinTransactionTypeEnum;
import com.xs.core.config.mq.RabbitMQConfig;
import com.xs.core.model.coin.entity.UserCoinTransaction;
import com.xs.core.model.coin.msg.AirdropManagerMessage;
import com.xs.core.service.wallet.IUserWalletService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

/**
 * 空投消息消费者
 */
@Component
@Slf4j
public class AirdropMessageConsumer {
    @Autowired
    private IUserWalletService walletService;

    @RabbitListener(queues = RabbitMQConfig.AIRDROP_QUEUE)
    public void handleComplete(Message m, Channel channel) throws IOException {
        String msg = new String(m.getBody());
        AirdropManagerMessage airdropManagerMessage = JSON.parseObject(msg, AirdropManagerMessage.class);
        if (ObjUtil.isNull(airdropManagerMessage)) {
            // 消息为空,直接确认消息
            channel.basicAck(m.getMessageProperties().getDeliveryTag(), false);
            return;
        }
        try {
            if (airdropManagerMessage.getOperationType() == 0) {
                walletService.coinTransaction(CoinTransactionTypeEnum.ADD, CoinTransactionCategoryEnum.RESOURCE_PLACEMENT, "空投增加金币", "空投增加金币,用户id:" + airdropManagerMessage.getTargetUser(), new BigDecimal(airdropManagerMessage.getGoldCoinNum()), airdropManagerMessage.getTargetUser());
                // 新增
            } else if (airdropManagerMessage.getOperationType() == 1) {
                // 减去
                //先获取用户的空投数量
                List<UserCoinTransaction> transactionList = walletService.getCoinTransactionByType(airdropManagerMessage.getTargetUser(), CoinTransactionCategoryEnum.RESOURCE_PLACEMENT);
                BigDecimal reduce = BigDecimal.ZERO;
                BigDecimal amount = new BigDecimal(airdropManagerMessage.getGoldCoinNum());
                if (CollUtil.isNotEmpty(transactionList)) {
                    reduce = transactionList.stream().filter(trans -> ObjUtil.isNotNull(trans.getAmount()))
                            .map(transaction -> {
                                if (Objects.equals(transaction.getTransactionType(), CoinTransactionTypeEnum.ADD.getCode())) {
                                    // 增加
                                    return transaction.getAmount();
                                } else if (Objects.equals(transaction.getTransactionType(), CoinTransactionTypeEnum.SUBTRACT.getCode())) {
                                    // 消耗（负值）
                                    return transaction.getAmount().negate();
                                } else {
                                    // 未知类型，当作0处理
                                    return BigDecimal.ZERO;
                                }
                            })
                            .reduce(BigDecimal.ZERO, BigDecimal::add);
                }
                if (reduce.compareTo(amount) >= 0) {
                    //减去
                    walletService.coinTransaction(CoinTransactionTypeEnum.SUBTRACT, CoinTransactionCategoryEnum.RESOURCE_PLACEMENT, "空投减少金币", "空投减少金币,用户id:" + airdropManagerMessage.getTargetUser(), new BigDecimal(airdropManagerMessage.getGoldCoinNum()), airdropManagerMessage.getTargetUser());
                } else {
                    walletService.coinTransaction(CoinTransactionTypeEnum.SUBTRACT, CoinTransactionCategoryEnum.RESOURCE_PLACEMENT, "空投减少金币", "空投减少金币,用户id:" + airdropManagerMessage.getTargetUser() + ",用户空投金币不足,实际减去金额为" + reduce.toPlainString(), reduce, airdropManagerMessage.getTargetUser());
                }
            }
            channel.basicAck(m.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("空投消息处理失败,消息内容:{}", msg, e);
            channel.basicNack(m.getMessageProperties().getDeliveryTag(), false, true);
        }
    }
}
