package com.xs.core.service.Impl.user;

import cn.dev33.satoken.stp.SaLoginConfig;
import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.xs.core.common.constant.ConstantConfig;
import com.xs.core.common.content.UserContext;
import com.xs.core.common.enums.DisEnableStatusEnum;
import com.xs.core.common.validation.CheckUtils;
import com.xs.core.model.user.entity.User;
import com.xs.core.model.user.req.AppLoginReq;
import com.xs.core.model.wallet.entity.UserWallet;
import com.xs.core.service.team.TeamShareService;
import com.xs.core.service.user.AppLoginService;
import com.xs.core.service.user.IUserService;
import com.xs.core.service.wallet.IUserWalletService;
import com.xs.core.utils.InviteCodeGenerator;
import com.xs.core.utils.SpringWebUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.concurrent.CompletableFuture;

@Service
@AllArgsConstructor
@Slf4j
public class AppLoginServiceImpl implements AppLoginService {
    private final IUserService userService;


    @Override
    public SaTokenInfo tgLogin(AppLoginReq loginReq) {
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(User::getTgId, loginReq.getId());
        User user = userService.getOne(queryWrapper);
        if (ObjUtil.isNull(user)) {
            user = userService.registerToSys(loginReq);
        }
        //校验是否被禁用
        checkUserStatus(user);
        return this.login(user);
    }

    /**
     * 检查用户状态
     *
     * @param user 用户信息
     */
    private void checkUserStatus(User user) {
        CheckUtils.throwIfEqual(DisEnableStatusEnum.DISABLE.getValue(), user.getDisableFlag(), "response.user.disable");
    }

    private SaTokenInfo login(User user) {
        Long userId = user.getId();
        user.setLoginTime(LocalDateTime.now());
        UserContext userContext = new UserContext();
        BeanUtils.copyProperties(user, userContext);
        userContext.setId(userId);
        // 记录登录时间以及ip等信息
        userContext.setExtraInfo(SpringWebUtils.getRequest());
        user.setLoginIp(userContext.getIp());
        user.setIpAddressConvert(userContext.getAddress());
        //先登出 防止同一个账号重复登录生成多个token
        StpUtil.logout(userId);
        // 登录并缓存用户信息
        StpUtil.login(userId, SaLoginConfig.setExtra(ConstantConfig.USER_CONTENT_EXTRA_KEY, JSON.toJSONString(userContext)));
        CompletableFuture.runAsync(() -> userService.updateById(user));
        return StpUtil.getTokenInfo();
    }
}
